/**************************************************************************************************
*                                                                                                 *
*  Class name:  OrderFormAppJDBC                                                                  *
*     Purpose:  This applet accesses the Sales Order, Sales Order Items, Product, Employee,       *
*               Customer, and Department tables.  OrderFormAppJDBC simulates a real order form    *
*               allowing users to add new orders to the tables mentioned above.  Additionally,    *
*               this applet allows you to search for orders from any field, edit an existing      *
*               order, and provides basic database functionality (e.g., First, Next, Delete)      *
*                                                                                                 *
*     Imports:  import java.applet.Applet;                                                        *
*               import java.awt.*;                                                                *
*               import java.lang.Thread;                                                          *
*               import java.lang.String;                                                          *
*               import symjava.sql.*;                                                             *
*               import java.util.Properties;                                                      *
*                                                                                                 *
*  Methods Include:  final void clear_record()                                                    *
*                    final void close_server()                                                    *
*                    final void connect_server()                                                  *
*                    final ResultSet generate_ResultSet()                                         *
*                    final boolean move()                                                         *
*                    final void save_info()                                                       *
*                    final void show_information()                                                *
*                    final void update_delete()                                                   *
*                    public boolean handleEvent()                                                 *
*                                                                                                 *
* Additional Notes:                                                                               *
* THIS SOFTWARE HAS BEEN COMPILED AND EXECUTED SUCCESSFULLY IN SPECIFIC SYMANTEC                  *
* ENVIRONMENTS, AND IS BEING PROVIDED ONLY AS SAMPLE CODE.                                        *
*                                                                                                 *
* SYMANTEC MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE,          *
* EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF               *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SYMANTEC SHALL NOT     *
* BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING, OR DISTRIBUTING *
* THIS SOFTWARE OR ITS DERIVATIVES.                                                               *
*                                                                                                 *
*  Please see the Sample Code Configuration  item in the Read Me file for additional              *
*  information about the sample database  used in this applet.
*
* Copyright (c) 1996 Symantec.  All Rights Reserved.
*
*                                                                                                 *
***************************************************************************************************/

import java.applet.Applet;
import java.awt.*;
import java.lang.Thread;
import java.lang.String;
import symjava.sql.*;
import symjava.math.BigDecimal;
import java.util.Properties;

public class OrderFormAppJDBC extends Applet {

    public void init() {
        // Initialize Properties
        props.put("user","dba");
        props.put("password","sql");

        // Set the font for everything but output
        this.setFont(hel_reg_font);

        // Initialize Background and Text Color
	    this.setBackground(Color.white);
	    this.setForeground(Color.black);

        super.init();

        // Initialize Buttons
        browse_button = new Button("Go to Browse Mode");
        add_button = new Button("Go to Add/Edit Mode");
        save_button = new Button("Save");
        delete_button = new Button("Delete");
        edit_button = new Button("Edit");
        clear_button = new Button("Clear Record");
        first_button = new Button("<<");
        next_button = new Button(">");
        search_button = new Button("Search");

        add_item_button = new Button("Add Item");
        edit_item_button = new Button("Update Item");
        delete_item_button = new Button("Delete Item");
        clear_item_button = new Button("Clear Item");
        quit_button = new Button("Quit");
        confirm_button = new Button("Confirm Action");

        // Initialize Choices
        empl_dept = new Choice();
        empl_dept.addItem("");
        empl_dept.addItem("Finance");
        empl_dept.addItem("Marketing");
        empl_dept.addItem("R & D");
        empl_dept.addItem("Sales");
        empl_dept.addItem("Shipping");
        empl_dept.select(0);

        req_region = new Choice();
        req_region.addItem("");
        req_region.addItem("Canada");
        req_region.addItem("Central");
        req_region.addItem("Eastern");
        req_region.addItem("South");
        req_region.addItem("Western");
        req_region.select(0);

	    // Initialize Labels
	    record_label = new Label("of");
	    record_label.setFont(hel_reg_14_font);

	    req_date_label = new Label("Purchase Date:");
	    req_date_label.setFont(hel_reg_11_font);

	    req_fin_label = new Label("Finance Code:");
	    req_fin_label.setFont(hel_reg_11_font);

	    req_region_label = new Label("Region:");
	    req_region_label.setFont(hel_reg_11_font);

	    cust_first_label = new Label("First Name:");
	    cust_first_label.setFont(hel_reg_11_font);

	    cust_last_label = new Label("Last Name:");
	    cust_last_label.setFont(hel_reg_11_font);

	    cust_company_label = new Label("Company:");
	    cust_company_label.setFont(hel_reg_11_font);

	    cust_addr_label = new Label("Address:");
	    cust_addr_label.setFont(hel_reg_11_font);

        cust_phone_label = new Label("Phone:");
        cust_phone_label.setFont(hel_reg_11_font);

        cust_city_label = new Label("City:");
        cust_city_label.setFont(hel_reg_11_font);

        cust_state_label = new Label("State:");
        cust_state_label.setFont(hel_reg_11_font);

        cust_zip_label = new Label("Zip Code:");
        cust_zip_label.setFont(hel_reg_11_font);

        empl_first_label = new Label("First Name:");
        empl_first_label.setFont(hel_reg_11_font);

        empl_last_label = new Label("Last Name:");
        empl_last_label.setFont(hel_reg_11_font);

        empl_dept_label = new Label("Department:");
        empl_dept_label.setFont(hel_reg_11_font);

        prod_line_label = new Label("Number:");
        prod_line_label.setFont(hel_reg_11_font);

        prod_id_label = new Label("ID #:");
        prod_id_label.setFont(hel_reg_11_font);

        prod_quan_label = new Label("Quantity:");
        prod_quan_label.setFont(hel_reg_11_font);

        prod_ship_label = new Label("Ship Date:");
        prod_ship_label.setFont(hel_reg_11_font);

        // Initialize Strings
        current_status_bar = new String();

	    // Initialize Textfields
	    status_bar = new TextField("Ready. System messages go here.", 40);
	    status_bar.setFont(hel_reg_14_font);
	    status_bar.setBackground(Color.black);
	    status_bar.setForeground(Color.orange);
	    status_bar.setEditable(false);

	    req_total = new TextField();
	    req_total.setFont(hel_reg_14_font);
	    req_total.setBackground(Color.black);
	    req_total.setForeground(Color.orange);
	    req_total.setEditable(false);

        req_current = new TextField();
	    req_current.setFont(hel_reg_14_font);
	    req_current.setBackground(Color.black);
	    req_current.setForeground(Color.orange);
	    req_current.setEditable(false);

	    req_num = new TextField();
	    req_num.setBackground(Color.black);
	    req_num.setForeground(Color.orange);
	    req_date = new TextField();
	    req_fin = new TextField("r1", 2);
	    req_fin.setEditable(false);
	    req_fin.setBackground(Color.lightGray);

	    cust_num = new TextField();
	    cust_first = new TextField();
        cust_last = new TextField();
        cust_company = new TextField();
        cust_addr = new TextField();
        cust_phone = new TextField();
        cust_city = new TextField();
        cust_state = new TextField();
        cust_zip = new TextField();

        empl_num = new TextField(15);
        empl_first = new TextField(20);
        empl_last = new TextField(35);

        prod_header = new TextField("#      UnitPrice  Quantity    Shipping      Product Information");
        prod_header.setFont(cour_reg_font);
        prod_header.setEditable(false);

        prod_line = new TextField();
        prod_id = new TextField();
        prod_quan = new TextField();
        prod_ship = new TextField();

        prod_mask = new TextField();
        prod_mask.disable();
        prod_mask.setBackground(Color.black);
        prod_mask.setEditable(false);

        // Initialize TextArea
        order_list = new TextArea();
        order_list.setFont(cour_reg_font);
        order_list.setEditable(false);

        this.setLayout(null);
        addNotify();
        resize(insets().left + insets().right + 540, insets().top + insets().bottom + 540);

        // Add header stuff
        add(req_current);
        req_current.reshape(insets().left+150, insets().top+50, 51, 24);

        add(record_label);
        record_label.reshape(insets().left+192, insets().top+50, 25, 24);

        add(req_total);
        req_total.reshape(insets().left+219, insets().top+50, 51, 24);

        add(browse_button);
        browse_button.reshape(insets().left, insets().top+75, 125, 20);

        add(add_button);
        add_button.reshape(insets().left, insets().top+75, 125, 20);

        add(save_button);
        save_button.reshape(insets().left + 127, insets().top+75, 55, 20);

        add(delete_button);
        delete_button.reshape(insets().left + 127, insets().top+75, 55, 20);

        add(edit_button);
        edit_button.reshape(insets().left+184, insets().top+75, 62, 20);

        add(first_button);
        first_button.reshape(insets().left+184, insets().top+75, 30, 20);

        add(next_button);
        next_button.reshape(insets().left+216, insets().top+75, 30, 20);

        add(search_button);
        search_button.reshape(insets().left+248, insets().top+75, 96, 20);

        add(clear_button);
        clear_button.reshape(insets().left+346, insets().top+75, 96, 20);

        add(quit_button);
        quit_button.reshape(insets().left+444, insets().top+75, 96, 20);

        add(status_bar);
        status_bar.reshape(insets().left+2, insets().top+96, 537, 23);

        // Add Requisition stuff
        add(req_num);
        req_num.reshape(insets().left+475,insets().top+121, 60, 19);

        add(req_date_label);
        req_date_label.reshape(insets().left, insets().top+145, 85, 15);
        add(req_date);
        req_date.reshape(insets().left+87, insets().top+144, 90, 19);

        add(req_fin_label);
        req_fin_label.reshape(insets().left+190, insets().top+145, 80, 15);
        add(req_fin);
        req_fin.reshape(insets().left+272, insets().top+144, 60, 19);

        add(req_region_label);
        req_region_label.reshape(insets().left+390, insets().top+145, 50, 15);
        add(req_region);
        req_region.reshape(insets().left+440, insets().top+142, 70, 16);

        // Add Customer stuff
        add(cust_num);
        cust_num.reshape(insets().left+475, insets().top+166, 60, 19);

        add(cust_first_label);
        cust_first_label.reshape(insets().left, insets().top+190, 66, 15);
        add(cust_first);
        cust_first.reshape(insets().left+68, insets().top+189, 85, 19);

        add(cust_last_label);
        cust_last_label.reshape(insets().left+150, insets().top+190, 64, 15);
        add(cust_last);
        cust_last.reshape(insets().left+216, insets().top+189, 90, 19);

        add(cust_company_label);
        cust_company_label.reshape(insets().left+323, insets().top+190, 64, 15);
        add(cust_company);
        cust_company.reshape(insets().left+387, insets().top+189, 150, 19);

        add(cust_addr_label);
        cust_addr_label.reshape(insets().left, insets().top+212, 66, 15);
        add(cust_addr);
        cust_addr.reshape(insets().left+68, insets().top+211, 200, 19);

        add(cust_phone_label);
        cust_phone_label.reshape(insets().left+394, insets().top+212, 45, 15);
        add(cust_phone);
        cust_phone.reshape(insets().left+442, insets().top+211, 95, 19);

        add(cust_city_label);
        cust_city_label.reshape(insets().left, insets().top+234, 66, 15);
        add(cust_city);
        cust_city.reshape(insets().left+68, insets().top+233, 90, 19);

        add(cust_state_label);
        cust_state_label.reshape(insets().left+155, insets().top+234, 40, 15);
        add(cust_state);
        cust_state.reshape(insets().left+197, insets().top+233, 40, 19);

        add(cust_zip_label);
        cust_zip_label.reshape(insets().left+235, insets().top+234, 58, 15);
        add(cust_zip);
        cust_zip.reshape(insets().left+295, insets().top+233, 60, 19);

        // Add Employee stuff
        add(empl_num);
        empl_num.reshape(insets().left+475, insets().top+255, 60, 19);

        add(empl_first_label);
        empl_first_label.reshape(insets().left, insets().top+279, 66, 15);
        add(empl_first);
        empl_first.reshape(insets().left+68, insets().top+278, 85, 19);

        add(empl_last_label);
        empl_last_label.reshape(insets().left+150, insets().top+279, 64, 15);
        add(empl_last);
        empl_last.reshape(insets().left+216, insets().top+278, 90, 19);

        add(empl_dept_label);
        empl_dept_label.reshape(insets().left+323, insets().top+279, 68, 15);
        add(empl_dept);
        empl_dept.reshape(insets().left+395, insets().top+278, 142, 19);

        // Add Product stuff
        add(prod_header);
        prod_header.reshape(insets().left+2, insets().top+323, 536, 22);

        add(order_list);
        order_list.reshape(insets().left+2, insets().top+347, 536, 146);

        add(prod_mask);
        prod_mask.reshape(insets().left, insets().top+495, 538, 47);

        add(prod_line_label);
        prod_line_label.reshape(insets().left, insets().top+496, 50, 15);
        add(prod_line);
        prod_line.reshape(insets().left+68, insets().top+495, 50, 19);

        add(prod_id_label);
        prod_id_label.reshape(insets().left+134, insets().top+496, 33, 15);
        add(prod_id);
        prod_id.reshape(insets().left+169, insets().top+495, 50, 19);

        add(prod_quan_label);
        prod_quan_label.reshape(insets().left+230, insets().top+496, 55, 15);
        add(prod_quan);
        prod_quan.reshape(insets().left+287, insets().top+495, 50, 19);

        add(prod_ship_label);
        prod_ship_label.reshape(insets().left+348, insets().top+496, 60, 15);
        add(prod_ship );
        prod_ship.reshape(insets().left+410, insets().top+495, 90, 19);

        add(add_item_button);
        add_item_button.reshape(insets().left, insets().top+516, 80, 20);

        add(edit_item_button);
        edit_item_button.reshape(insets().left+88, insets().top+516, 98, 20);

        add(delete_item_button);
        delete_item_button.reshape(insets().left+196, insets().top+516, 107, 20);

        add(confirm_button);
        confirm_button.reshape(insets().left+313, insets().top+516, 120, 20);

        add(clear_item_button);
        clear_item_button.reshape(insets().left+443, insets().top+516, 97, 20);

        // Display UI to the screen
        show();
        disable_buttons();

        // Try to connect to the server.  The connection will be keep up for the whole
        // duration of the session that the applet is used.
        connect_server();
        if (good_connection) {
            try {
                message("Establishing connection...");
                // create a statement to be executed from the connection
                stmt = server.createStatement();
            }
            catch (SQLException create_error) {
                message("Exception caught!  Could not create multiple statements!");
                sleep();
            }
        }
        else {
            message("Could not open server!");
            sleep();
        }

        // set up initial state
        set_browse_mode();
        order_list.setText("OrderFormAppJDBC v0.23\n");
        order_list.appendText("Please note that there is no error checking for input.");
    }

    // already_exist()
    final boolean already_exist(String from_clause, String table_clause, String identifier_clause) {
        String prod_SQL = "SELECT count(*) FROM "+from_clause+" WHERE "+table_clause+" = '" + identifier_clause + "'";
        int total = 0;

        try {
            message("Checking if record already exists...");
            rs = generate_ResultSet(prod_SQL, rs, stmt, true);
            total = rs.getInt(1);
            rs.close();
        }
        catch (SQLException create_error) {
            message("Exception caught!  Could not check if record exists!");
            sleep();
        }

        if (total == 1) { return true; }
        else if (total == 0) { return false; }
        else {
            message("Data integrity volation has been detected!");
            sleep();
        }
        return false;
    }

    // clear_item()
    final void clear_item() {
        prod_id.setText("");
        prod_quan.setText("");
        prod_ship.setText("");
    }

    // clear_record()
    final void clear_record() {
        if (mode != 1) { req_num.setText(""); req_date.setText(""); }
        if (mode == 2) { set_browse_mode(); }

        req_region.select(0);
        cust_num.setText("");
        cust_first.setText("");
        cust_last.setText("");
        cust_addr.setText("");
        cust_company.setText("");
        cust_city.setText("");
        cust_state.setText("");
        cust_zip.setText("");
        cust_phone.setText("");
        empl_num.setText("");
        empl_first.setText("");
        empl_last.setText("");
        empl_dept.select(0);
        order_list.setText("");

        clear_item();
    }

    // close_server()
    final void close_server() {
        message("Attempting to close server "+server_url+"...");
        try {
            server.close();
            good_connection = false;
            message("Server closed successfully!");
        }
        catch (Exception close_error) {
            message("Exception caught! Could not close server!");
            sleep();
        }
    }

    // connect_server()
    final void connect_server() {
        message("Attempting to open server "+server_url+"...");
        try {
            // create a specific driver for use with JDBC
            dbaw_driver = (Driver)Class.forName(dbaw_driver_name).newInstance();

            // connect to server given by server_url and props
            server = dbaw_driver.connect(server_url, props);
            //server.setAutoClose(false);
            good_connection = true;
            message("Server opened successfully!");
        }
        catch (InstantiationException instantiation_error) { }
        catch (ClassNotFoundException classnotfound_error) { }
        catch (IllegalAccessException illegalaccess_error) { }
        catch (SQLException open_error) {
            message("Exception caught! Could not open server!");
            sleep();
        }
    }

    // disable_buttons()
    final void disable_buttons() {
        browse_button.disable();
        add_button.disable();
        delete_button.disable();
        edit_button.disable();
        save_button.disable();
        clear_button.disable();
        first_button.disable();
        next_button.disable();
        search_button.disable();
        add_item_button.disable();
        edit_item_button.disable();
        delete_item_button.disable();
        clear_item_button.disable();
        confirm_button.disable();
    }

    // generate_ResultSet()
    final ResultSet generate_ResultSet(String target_SQL, ResultSet target_RS, Statement target_Stmt, boolean get_record) {

        try {
            message("Grabbing results...");
            target_RS = target_Stmt.executeQuery(target_SQL);
            message("Ready.");
        }
        catch (Exception search_error) {
            message("Could not find any hits!");
        }

        if (get_record) {
            try { target_RS.next(); }
            catch (SQLException next_error) {
                message("Exception caught!  Could not grab next record!");
                sleep();
            }
        }

        return (target_RS);
    }

    // generate_SQL()
    final String generate_SQL(int current_mode, boolean is_count) {
        String select = "SELECT sales_order.id, cust_id, order_date, region, sales_rep,"
            + " fname, lname, customer.address, customer.city, customer.state, zip,"
            + " customer.phone, company_name, emp_fname, emp_lname, employee.dept_id,"
            + " dept_name, sales_order_items.id, line_id, prod_id, sales_order_items.quantity,"
            + " ship_date, name, description, size, color, unit_price";
        String select_count = "SELECT count(*)";
        String from = " FROM sales_order, customer, employee, department, sales_order_items, product";
        String from_count = " FROM sales_order";
        String where = " WHERE customer.id = cust_id AND emp_id = sales_rep AND department.dept_id = employee.dept_id AND product.id = prod_id";
        String where_count = "";

        boolean req_num_exist = false, req_date_exist = false;
        boolean first_req = true, first_cust = true, first_empl = true, first_dept = true;
        boolean req_region_exist = false, cust_num_exist = false, empl_num_exist = false;
        boolean cust_first_exist = false, cust_last_exist = false, cust_company_exist = false;
        boolean cust_addr_exist = false, cust_city_exist = false, cust_state_exist = false;
        boolean cust_zip_exist = false, cust_phone_exist = false, empl_dept_exist = false;
        boolean empl_first_exist = false, empl_last_exist = false;

        boolean req_num_wild = has_wildcard(req_num);
        boolean req_date_wild = has_wildcard(req_date);
        boolean cust_num_wild = has_wildcard(cust_num);
        boolean empl_num_wild = has_wildcard(empl_num);
        boolean cust_first_wild = has_wildcard(cust_first);
        boolean cust_last_wild = has_wildcard(cust_last);
        boolean cust_company_wild = has_wildcard(cust_company);
        boolean cust_addr_wild = has_wildcard(cust_addr);
        boolean cust_city_wild = has_wildcard(cust_city);
        boolean cust_state_wild = has_wildcard(cust_state);
        boolean cust_zip_wild = has_wildcard(cust_zip);
        boolean cust_phone_wild = has_wildcard(cust_phone);
        boolean empl_first_wild = has_wildcard(empl_first);
        boolean empl_last_wild = has_wildcard(empl_last);

        if (req_num.getText().length() != 0) req_num_exist = true;
        if (req_date.getText().length() != 0) req_date_exist = true;
        if (req_region.getSelectedIndex() != 0) req_region_exist = true;
        if (cust_num.getText().length() != 0) cust_num_exist = true;
        if (empl_num.getText().length() != 0) empl_num_exist = true;

        if (cust_first.getText().length() != 0) cust_first_exist = true;
        if (cust_last.getText().length() != 0) cust_last_exist = true;
        if (cust_company.getText().length() != 0) cust_company_exist = true;
        if (cust_addr.getText().length() != 0) cust_addr_exist = true;
        if (cust_city.getText().length() != 0) cust_city_exist = true;
        if (cust_state.getText().length() != 0) cust_state_exist = true;
        if (cust_zip.getText().length() != 0) cust_zip_exist = true;
        if (empl_first.getText().length() != 0) empl_first_exist = true;
        if (empl_last.getText().length() != 0) empl_last_exist = true;
        if (empl_dept.getSelectedIndex() != 0) empl_dept_exist = true;

        message("Dynamically generating SQL statement...");

        if (req_num_exist || req_date_exist || req_region_exist || cust_num_exist || empl_num_exist) {
            if ( (req_num_exist) && (!req_num_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE sales_order.id = '" + req_num.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND sales_order.id = '" + req_num.getText() + "'");
                }
                where = where.concat(" AND sales_order.id = '" + req_num.getText() + "'");
            }
            else if ( (req_num_exist) && (req_num_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE sales_order.id LIKE '" + req_num.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND sales_order.id LIKE '" + req_num.getText() + "'");
                }
                where = where.concat(" AND sales_order.id LIKE '" + req_num.getText() + "'");
            }

            if ( (cust_num_exist) && (!cust_num_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE cust_id = '" + cust_num.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND cust_id = '" + cust_num.getText() + "'");
                }
                where = where.concat(" AND cust_id = '" + cust_num.getText() + "'");
            }
            else if ( (cust_num_exist) && (cust_num_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE cust_id LIKE '" + cust_num.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND cust_id LIKE '" + cust_num.getText() + "'");
                }
                where = where.concat(" AND cust_id LIKE '" + cust_num.getText() + "'");
            }

            if ( (req_date_exist) && (!req_date_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE order_date = '" + req_date.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND order_date = '" + req_date.getText() + "'");
                }
                where = where.concat(" AND order_date = '" + req_date.getText() + "'");
            }
            else if ( (req_date_exist) && (req_date_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE order_date LIKE '" + req_date.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND order_date LIKE '" + req_date.getText() + "'");
                }
                where = where.concat(" AND order_date LIKE '" + req_date.getText() + "'");
            }

            if (req_region_exist) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE region = '" + req_region.getSelectedItem() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND region = '" + req_region.getSelectedItem() + "'");
                }
                where = where.concat(" AND region = '" + req_region.getSelectedItem() + "'");
            }

            if ( (empl_num_exist) && (!empl_num_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE sales_rep = '" + empl_num.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND sales_rep= '" + empl_num.getText() + "'");
                }
                where = where.concat(" AND sales_rep = '" + empl_num.getText() + "'");
            }
            else if ( (empl_num_exist) && (empl_num_wild) ) {
                if (first_req) {
                    where_count = where_count.concat(" WHERE sales_rep LIKE '" + empl_num.getText() + "'");
                    first_req = false;
                }
                else {
                    where_count = where_count.concat(" AND sales_rep LIKE '" + empl_num.getText() + "'");
                }
                where = where.concat(" AND sales_rep LIKE '" + empl_num.getText() + "'");
            }
        }

            if (cust_first_exist || cust_last_exist || cust_company_exist || cust_addr_exist
            || cust_city_exist || cust_state_exist || cust_zip_exist || empl_first_exist
            || empl_last_exist || empl_dept_exist) {
                if (cust_first_exist) {
                    if (!from_count.endsWith(", customer")) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_first_wild) {
                        where_count = where_count.concat(" AND fname = '" + cust_first.getText() + "'");
                        where = where.concat(" AND fname = '" + cust_first.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND fname LIKE '" + cust_first.getText() + "'");
                        where = where.concat(" AND fname LIKE '" + cust_first.getText() + "'");
                    }
                }

                if (cust_last_exist) {
                    if (!from_count.endsWith(", customer")) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_last_wild) {
                        where_count = where_count.concat(" AND lname = '" + cust_last.getText() + "'");
                        where = where.concat(" AND lname = '" + cust_last.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND lname LIKE '" + cust_last.getText() + "'");
                        where = where.concat(" AND lname LIKE '" + cust_last.getText() + "'");
                    }
                }

                if (cust_company_exist) {
                    if (!from_count.endsWith(", customer")) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_company_wild) {
                        where_count = where_count.concat(" AND company_name = '" + cust_company.getText() + "'");
                        where = where.concat(" AND company_name = '" + cust_company.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND company_name LIKE '" + cust_company.getText() + "'");
                        where = where.concat(" AND company_name LIKE '" + cust_company.getText() + "'");
                    }
                }

                if (cust_addr_exist) {
                    if (!from_count.endsWith(", customer")) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_addr_wild) {
                        where_count = where_count.concat(" AND customer.address = '" + cust_addr.getText() + "'");
                        where = where.concat(" AND customer.address = '" + cust_addr.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND customer.address LIKE '" + cust_addr.getText() + "'");
                        where = where.concat(" AND customer.address LIKE '" + cust_addr.getText() + "'");
                    }
                }

                if (cust_city_exist) {
                    if (!(from_count.endsWith(", customer"))) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_city_wild) {
                        where_count = where_count.concat(" AND customer.city = '" + cust_city.getText() + "'");
                        where = where.concat(" AND customer.city = '" + cust_city.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND customer.city LIKE '" + cust_city.getText() + "'");
                        where = where.concat(" AND customer.city LIKE '" + cust_city.getText() + "'");
                    }
                }

                if (cust_state_exist) {
                    if (!(from_count.endsWith(", customer"))) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_state_wild) {
                        where_count = where_count.concat(" AND customer.state = '" + cust_state.getText() + "'");
                        where = where.concat(" AND customer.state = '" + cust_state.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND customer.state LIKE '" + cust_state.getText() + "'");
                        where = where.concat(" AND customer.state LIKE '" + cust_state.getText() + "'");
                    }
                }

                if (cust_zip_exist) {
                    if (!from_count.endsWith(", customer")) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_zip_wild) {
                        where_count = where_count.concat(" AND zip = '" + cust_zip.getText() + "'");
                        where = where.concat(" AND zip = '" + cust_zip.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND zip LIKE '" + cust_zip.getText() + "'");
                        where = where.concat(" AND zip LIKE '" + cust_zip.getText() + "'");
                    }
                }

                if (cust_phone_exist) {
                    if (!from_count.endsWith(", customer")) {
                        from_count = from_count.concat(", customer");
                    }
                    if ((first_req) && (first_cust)) {
                        where_count = where_count.concat(" WHERE customer.id = cust_id");
                        first_cust = false;
                    }
                    else if ((!first_req) && (first_cust)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "customer.id = cust_id AND ");
                        where_count = string_buffer.toString();
                        first_cust = false;
                    }

                    if (!cust_phone_wild) {
                        where_count = where_count.concat(" AND customer.phone = '" + cust_phone.getText() + "'");
                        where = where.concat(" AND customer.phone = '" + cust_phone.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND customer.phone LIKE '" + cust_phone.getText() + "'");
                        where = where.concat(" AND customer.phone LIKE '" + cust_phone.getText() + "'");
                    }
                }

                if (empl_first_exist) {
                    if (!from_count.endsWith(", employee")) {
                        from_count = from_count.concat(", employee");
                    }
                    if ((first_req) && (first_cust) && (first_empl)) {
                        where_count = where_count.concat(" WHERE emp_id = sales_rep");
                        first_empl = false;
                    }
                    else if (((!first_req) || (!first_cust)) && (first_empl)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "emp_id = sales_rep AND ");
                        where_count = string_buffer.toString();
                        first_empl = false;
                    }

                    if (!empl_first_wild) {
                        where_count = where_count.concat(" AND emp_fname = '" + empl_first.getText() + "'");
                        where = where.concat(" AND emp_fname = '" + empl_first.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND emp_fname LIKE '" + empl_first.getText() + "'");
                        where = where.concat(" AND emp_fname LIKE '" + empl_first.getText() + "'");
                    }
                }

                if (empl_last_exist) {
                    if (!from_count.endsWith(", employee")) {
                        from_count = from_count.concat(", employee");
                    }
                    if ((first_req) && (first_cust) && (first_empl)) {
                        where_count = where_count.concat(" WHERE emp_id = sales_rep");
                        first_empl = false;
                    }
                    else if (((!first_req) || (!first_cust)) && (first_empl)) {
                        StringBuffer string_buffer = new StringBuffer(where_count);
                        string_buffer.insert(7, "emp_id = sales_rep AND ");
                        where_count = string_buffer.toString();
                        first_empl = false;
                    }

                    if (!empl_last_wild) {
                        where_count = where_count.concat(" AND emp_lname = '" + empl_last.getText() + "'");
                        where = where.concat(" AND emp_lname = '" + empl_last.getText() + "'");
                    }
                    else {
                        where_count = where_count.concat(" AND emp_lname LIKE '" + empl_last.getText() + "'");
                        where = where.concat(" AND emp_lname LIKE '" + empl_last.getText() + "'");
                    }
                }

                if (empl_dept_exist) {
                    String dept_SQL = "SELECT dept_id FROM department WHERE dept_name = '" + empl_dept.getSelectedItem() + "'";
                    int dept_id = 0;
                    boolean dept_found = false;

                    try {
                        message("Grabbing department ID number...");
                        rs = generate_ResultSet(dept_SQL, rs, stmt, true);
                        dept_id = rs.getInt(1);
                        message("Searching department ID...");
                        dept_found = true;
                        rs.close();
                    }
                    catch (SQLException dept_error) {
                        empl_dept.select(0);
                    }

                    if (dept_found) {
                        if (!from_count.endsWith(", employee")) {
                            from_count = from_count.concat(", employee");
                        }
                        if ((first_req) && (first_cust) && (first_empl) && (first_dept)) {
                            where_count = where_count.concat(" WHERE emp_id = sales_rep");
                            first_dept = false;
                        }
                        else if (((!first_req) || (!first_cust) || (!first_empl)) && (first_empl)) {
                            StringBuffer string_buffer = new StringBuffer(where_count);
                            string_buffer.insert(7, "emp_id = sales_rep AND ");
                            where_count = string_buffer.toString();
                            first_dept = false;
                        }

                        where_count = where_count.concat(" AND employee.dept_id = " + String.valueOf(dept_id));
                        where = where.concat(" AND employee.dept_id = " + String.valueOf(dept_id));
                    }
                }
        }
        // assemble and return the SQL statement
        where = where.concat(" AND sales_order_items.id = sales_order.id");

        if (is_count) { return(select_count + from_count + where_count); }
        return (select + from + where + " ORDER BY 1, 18, 19 ASC");
    }

    // get_cust_info()
    final void get_cust_info() {
        String cust_SQL = "SELECT fname, lname, address, city, state, zip, phone, company_name FROM customer WHERE id = '"+cust_num.getText().trim()+"'";

        try {
            message("Opening customer records...");
            rs = generate_ResultSet(cust_SQL, rs, stmt, true);
            message("Displaying results...");

            if (rs.getString(1) != null) cust_first.setText(rs.getString(1));
            else cust_first.setText("");
            if (rs.getString(2) != null) cust_last.setText(rs.getString(2));
            else cust_last.setText("");
            if (rs.getString(3) != null) cust_addr.setText(rs.getString(3));
            else cust_addr.setText("");
            if (rs.getString(4) != null) cust_city.setText(rs.getString(4));
            else cust_city.setText("");
            if (rs.getString(5) != null) cust_state.setText(rs.getString(5));
            else cust_state.setText("");
            if (rs.getString(6) != null) cust_zip.setText(rs.getString(6));
            else cust_zip.setText("");
            if (rs.getString(7) != null) cust_phone.setText(rs.getString(7));
            else cust_phone.setText("");
            if (rs.getString(8) != null) cust_company.setText(rs.getString(8));
            else cust_company.setText("");

            message("Ready. [Add Mode]");
            highlight(empl_num);
            rs.close();
        }
        catch (SQLException validity_error) {
            message("The customer ID you entered was not valid.  Try again.");
            highlight(cust_num);
        }
    }

    // get_edit_info()
    final void get_edit_info() {
        boolean worked = false;
        String select = "SELECT sales_order.id, cust_id, order_date, region, sales_rep,"
            + " fname, lname, customer.address, customer.city, customer.state, zip,"
            + " customer.phone, company_name, emp_fname, emp_lname, employee.dept_id,"
            + " dept_name, sales_order_items.id, line_id, prod_id, sales_order_items.quantity,"
            + " ship_date, name, description, size, color, unit_price";
        String from = " FROM sales_order, customer, employee, department, sales_order_items, product";
        String where = " WHERE customer.id = cust_id AND emp_id = sales_rep AND"
            + " department.dept_id = employee.dept_id AND product.id = prod_id AND"
            + " sales_order_items.id = sales_order.id AND sales_order.id = " + req_num.getText().trim();

        req_num.setEditable(false);

        message("Grabbing record from database...");
        rs = generate_ResultSet(select+from+where, rs, stmt, true);
        message("Displaying results...");

        worked = move(3);
        if (worked) {
            req_num.setBackground(Color.black);
            req_num.setForeground(Color.orange);
            highlight(prod_line);
            set_edit_item(3);
            message("Ready. [Edit Mode]");
            try { rs.close(); }
            catch (SQLException close_error) { message("Exception caught! Could not close record!"); }
            record_saved = true;
            update_prod_line(true);
        }
        else {
            message("The record number you entered was not valid.  Try again.");
            req_num.setEditable(true);
            highlight(req_num);
        }
    }

    // get_empl_info()
    final void get_empl_info() {
        String empl_SQL = "SELECT emp_fname, emp_lname, employee.dept_id, dept_name FROM employee, department WHERE department.dept_id = employee.dept_id AND emp_id = '"+empl_num.getText().trim()+"'";

        try {
            message("Opening employee records...");
            rs = generate_ResultSet(empl_SQL, rs, stmt, true);
            message("Displaying results...");

            if (rs.getString(1) != null) empl_first.setText(rs.getString(1));
            else empl_first.setText("");
            if (rs.getString(2) != null) empl_last.setText(rs.getString(2));
            else empl_last.setText("");
            if (rs.getString(4) != null) empl_dept.select(rs.getString(4));
            else empl_dept.select(0);

            message("Ready. [Add Mode]");
            highlight(prod_id);
            rs.close();
        }
        catch (SQLException validity_error) {
            message("The employee ID you entered was not valid.  Try again.");
            highlight(empl_num);
        }
    }

    // get_last_req_num()
    final void get_last_req_num() {
        String last_SQL = "SELECT MAX(sales_order.id) FROM sales_order";

        current_item_pos = 0;

        try {
            message("Getting last record of database...");
            rs = generate_ResultSet(last_SQL, rs, stmt, true);
            req_num.setText(String.valueOf(rs.getInt(1)+1));
            rs.close();
        }
        catch (SQLException get_int_error) {
            message("Exception caught!  Could not grab integer for Number!");
            sleep();
        }

        message("Getting today's date...");
        java.util.Date todays_date = new java.util.Date();
        Date sql_date = new Date(todays_date.getYear(), todays_date.getMonth(), todays_date.getDate());
        req_date.setText(sql_date.toString());
        req_date.setEditable(false);

        current_item_pos++;
        prod_line.setText(String.valueOf(current_item_pos));

        req_region.requestFocus();
        message("Ready. [Add Mode]");
    }

    // get_prod_info()
    final void get_prod_info() {
        String prod_SQL = "SELECT count(*) FROM product WHERE id = '" + prod_id.getText().trim() + "'";
        int total;

        try {
            message("Executing temporary statement...");
            rs = generate_ResultSet(prod_SQL, rs, stmt, true);
            total = rs.getInt(1);

            if (total == 1) {
                message("Ready. [Add Mode]");
                if (mode == 1) { highlight(prod_quan); }
                else if (mode == 4) {
                    prod_quan.requestFocus();
                    prod_quan.selectAll();
                }
            }
            else {
                message("The product ID you entered was not valid.  Try again.");
                highlight(prod_id);
            }

            rs.close();
        }
        catch (SQLException create_error) {
            message("Exception caught!  Could not create temporary statement!");
            sleep();
        }
    }

    // get_prod_item()
    final void get_prod_item() {
        String prod_item_SQL = "SELECT line_id, prod_id, quantity, ship_date FROM sales_order_items WHERE id = '" + req_num.getText().trim() + "' AND line_id = '" + prod_line.getText().trim() + "'";

        message("Grabbing product information...");
        rs = generate_ResultSet(prod_item_SQL, rs, stmt, false);

        try {
            rs.next();

            prod_id.setText("                        ");
            prod_quan.setText("                        ");
            prod_ship.setText("                        ");

            prod_line.setText(String.valueOf(rs.getString(1)));
            prod_id.setText(String.valueOf(rs.getString(2)));
            prod_quan.setText(String.valueOf(rs.getString(3)));
            prod_ship.setText(String.valueOf(rs.getString(4)));
            if (mode == 2) {
                message("Please confirm requested action...");
                confirm_button.enable();
                confirm_button.requestFocus();
            }
            if (mode == 3) set_editing();
        }
        catch (SQLException next_error) {
            message("Order not found!");
            if (mode == 2) { highlight(prod_line); }
            else if (mode == 3) {
                current_item_pos = Integer.valueOf(prod_line.getText()).intValue();
                update_prod_line(false);
            }
            else highlight(prod_line);
        }

        try { rs.close(); }
        catch (SQLException close_error) { message("Exception caught! Could not close record!"); }
    }

    // has_wildcard()
    final boolean has_wildcard(TextField target) {
        boolean return_value = false;

        target.setText(target.getText().trim().replace('\u0027', '_'));
        target.setText(target.getText().replace('*', '%').replace('?', '_'));

        if ((target.getText().indexOf("%", 0)) != -1) { return_value = true; }
        else if ((target.getText().indexOf("_", 0)) != -1) { return_value =  true; }
        else return_value = false;

        return return_value;
    }

    // highlight()
    final void highlight(TextField target) {
        target.setText("                        ");
        target.requestFocus();
        target.selectAll();
    }

    // message()
    final void message(String text) {
        current_status_bar = text;
        status_bar.setText(current_status_bar);
    }

    // move()
    final boolean move(int first_rec) {
        int current_record;

        try {
            if (first_rec == 1) { message("Grabbing first record..."); }
            else if (first_rec == 2) { message("Grabbing next record..."); }
            else if (first_rec == 3) { message("Grabbing data..."); }

            if (first_rec == 1) rs.next();
            current_record = rs.getInt(1);

            req_current.setText(String.valueOf(current_pos+1));

            message("Displaying record...");

            if (rs.getString(1) != null) req_num.setText(rs.getString(1));
            else req_num.setText("");
            if (rs.getString(2) != null) cust_num.setText(rs.getString(2));
            else cust_num.setText("");
            if (rs.getString(3) != null) req_date.setText(rs.getString(3));
            else req_date.setText("");
            if (rs.getString(4) != null) req_region.select(rs.getString(4));
            else req_region.select(0);
            if (rs.getString(5) != null) empl_num.setText(rs.getString(5));
            else empl_num.setText("");

            if (rs.getString(6) != null) cust_first.setText(rs.getString(6));
            else cust_first.setText("");
            if (rs.getString(7) != null) cust_last.setText(rs.getString(7));
            else cust_last.setText("");
            if (rs.getString(8) != null) cust_addr.setText(rs.getString(8));
            else cust_addr.setText("");
            if (rs.getString(9) != null) cust_city.setText(rs.getString(9));
            else cust_city.setText("");
            if (rs.getString(10) != null) cust_state.setText(rs.getString(10));
            else cust_state.setText("");
            if (rs.getString(11) != null) cust_zip.setText(rs.getString(11));
            else cust_zip.setText("");
            if (rs.getString(12) != null) cust_phone.setText(rs.getString(12));
            else cust_phone.setText("");
            if (rs.getString(13) != null) cust_company.setText(rs.getString(13));
            else cust_company.setText("");

            if (rs.getString(14) != null) empl_first.setText(rs.getString(14));
            else empl_first.setText("");
            if (rs.getString(15) != null) empl_last.setText(rs.getString(15));
            else empl_last.setText("");

            if (rs.getString(17) != null) empl_dept.select(rs.getString(17));
            else empl_dept.select(0);

            String padding = new String("                                                   ");
            String temp_string;
            BigDecimal subtotal = new BigDecimal ("0");
            BigDecimal sub_temp = new BigDecimal ("0");

            order_list.setText("");
            try {
                for (; rs.getInt(18) == current_record; rs.next()) {
                    message("Displaying ordered items");

                    temp_string = new String(rs.getString(19).concat(padding));
                    temp_string = temp_string.substring(0, 4);

                    BigDecimal price_temp = new BigDecimal(rs.getBigDecimal(27, 2).doubleValue());
                    BigDecimal quantity_temp = new BigDecimal(rs.getString(21));

                    order_list.appendText(temp_string+"   $"+price_temp.toString()+"\t     "
                        +rs.getString(21)+"\t     "+rs.getString(22)+"\t"
                        +rs.getString(23)+" - "+rs.getString(24)+"\n");
                    order_list.appendText("\t\t\t\t\tSize: "+rs.getString(25)+" - Color: "
                        +rs.getString(26)+"\n");
                    message("Calculating subtotal...");

                    sub_temp = price_temp.multiply(quantity_temp);
                    subtotal = subtotal.add(sub_temp);
                }
            }
            catch (SQLException order_list_error) { message("End of results reached."); }

            order_list.appendText("--------------------------------------------------------------------------\n");
            order_list.appendText("\t\t\t\t\tSubtotal: $"+subtotal+"\n");

            message("Ready.");

            if ((current_pos == (counter - 1)) && (current_pos == 0)) { nav_buttons(false, false); }
            else if (current_pos == (counter - counter)) { nav_buttons(false, true); }
            else if (current_pos == (counter - 1)) { nav_buttons(true, false); }
            else if (current_pos < (counter - 1)) { nav_buttons(true, true); }
            else {
                message("Problem with current_pos!  Exceeds counter!");
                return false;
            }
        }
        catch (SQLException next_error) {
            message("Exception caught!  Record grab unsuccessful!");
            return false;
        }
        return true;
    }

    // nav_buttons()
    final void nav_buttons(boolean one, boolean two) {
        if (one) {first_button.enable();}
        else {first_button.disable();}

        if (two) {next_button.enable();}
        else {next_button.disable();}
    }

    // paint()
    public void paint(Graphics g) {
        super.paint(g);

        // Company info
        g.setColor(Color.black);
        g.setFont(company_font);
        g.drawString("ADGGIL Systems.", 4, 25);

        g.setFont(hel_reg_font);
        g.drawString("123 Anywhere Street", 4, 40);
        g.drawString("AnyCity, State Zip", 4, 54);
        g.drawString("(xxx) 555 - 1234", 4, 68);

        // OrderForm info
        g.setFont(hel_bold_16_font);
        g.drawString("Purchase Requisition", 364, 18);

        g.setFont(hel_reg_font);
	    g.drawString("Complete all ungrayed portions of this form.", 290, 33);
	    g.drawString("Click on the form to refresh.", 379, 46);

        // Requisition info
        g.setColor(Color.black);
	    g.fillRect(2, 120, 537, 22);

	    g.setFont(hel_reg_14_font);
	    g.setColor(Color.white);
	    g.drawString("Requisition Information", 4, 135);
	    g.drawString("Number:", 420, 135);

	    // Customer info
	    g.setColor(Color.black);
	    g.fillRect(2, 165, 537, 22);

	    g.setFont(hel_reg_14_font);
	    g.setColor(Color.white);
	    g.drawString("Customer Information", 4, 180);
        g.drawString("ID:", 456, 180);

        // Employee info
        g.setColor(Color.black);
        g.fillRect(2, 254, 537, 22);

        g.setFont(hel_reg_14_font);
        g.setColor(Color.white);
        g.drawString("Employee Information", 4, 269);
        g.drawString("ID:", 456, 269);

        // Product info
        g.setColor(Color.black);
        g.fillRect(2, 299, 537, 22);

        g.setFont(hel_reg_14_font);
        g.setColor(Color.white);
        g.drawString("Product Information", 4, 314);
    }

    // save_info()
    final void save_info() {
        String save_SQL = "";
        boolean is_missing = true;

        if ((req_num.getText().trim().length() != 0) && (req_date.getText().trim().length() != 0) && (req_region.getSelectedIndex() != 0)
            && (cust_num.getText().trim().length() != 0) && (empl_num.getText().trim().length() != 0)) { is_missing = false; }
        else { message("Cannot save!  Empty fields were detected in record!"); }

        if (!is_missing) {
            if (!already_exist("sales_order", "id", req_num.getText().trim())) {
                save_SQL = "INSERT INTO sales_order (id, cust_id, order_date, fin_code_id, region, sales_rep) VALUES ("+req_num.getText().trim()+", "+cust_num.getText().trim()+", '"+req_date.getText().trim()+"', '"
                    +req_fin.getText().trim()+"', '"+req_region.getSelectedItem()+"', "+empl_num.getText().trim()+")";

                message("Saving record information...");

                try {
                    stmt.executeUpdate(save_SQL);
                    record_saved = true;
                    message("Record saved... press return or click on the Add Item button to same item.");
                }
                catch (SQLException save_error) { message("Problem with SQL Statement!  No records were inserted!"); }
            }
            else {
                message("This record already exists!");
                sleep();
            }
        }
    }

    // save_item_info()
    final void save_item_info() {
        String save_SQL = "";
        boolean is_missing = true;

        if ((req_num.getText().trim().length() != 0) && (prod_line.getText().trim().length() != 0)
            && (prod_id.getText().trim().length() != 0) && (prod_quan.getText().trim().length() != 0)
            && (prod_ship.getText().trim().length() != 0)) { is_missing = false; }
        else { message("Cannot save!  Empty fields were detected in item!"); }

        if (!is_missing) {
            if ( already_exist("sales_order", "id", req_num.getText().trim()) ) {
                save_SQL = "INSERT INTO sales_order_items (id, line_id, prod_id, quantity, ship_date) VALUES ("
                    +req_num.getText().trim()+", "+prod_line.getText().trim()+", "
                    +prod_id.getText().trim()+", "+prod_quan.getText().trim()+", '"
                    +prod_ship.getText().trim()+"')";

                message("Saving items...");

                try {
                    stmt.executeUpdate(save_SQL);
                    message("Item saved.");
                    clear_item();
                    current_item_pos++;
                    update_prod_line(true);
                    prod_line.setText(String.valueOf(current_item_pos));
                    if (current_item_pos > 1) {
                        edit_item_button.enable();
                        delete_item_button.enable();
                    }
                }
                catch (SQLException save_error) { message("Problem with SQL Statement!  No records were inserted!"); }
                show_order_list();
            }
            else {
                message("Record does not exist yet.  Please save the record first.");
                edit_item_button.requestFocus();
            }
        }
    }

    // set_add_mode()
    final void set_add_mode() {
        mode = 1;
        clear_record();
        set_modify_mode(true);
        get_last_req_num();
        confirm_button.setLabel("Confirm Action.");
    }

    // set_browse_mode()
    final void set_browse_mode() {
        mode = 0;
        record_saved = false;

        req_total.setText("");
        req_current.setText("");

        browse_button.disable();
        browse_button.hide();
        add_button.enable();
        add_button.show();
        save_button.disable();
        save_button.hide();
        edit_button.disable();
        edit_button.hide();
        delete_button.enable();
        delete_button.show();
        search_button.enable();
        clear_button.enable();
        clear_button.setLabel("Clear Record");
        first_button.disable();
        next_button.disable();

        req_total.show();
        record_label.show();
        req_current.show();

        req_num.setEditable(true);
        req_num.setBackground(Color.black);
        req_num.setForeground(Color.orange);
        req_date.setEditable(true);
        req_region.select(0);
        cust_first.setEditable(true);
        cust_last.setEditable(true);
        cust_addr.setEditable(true);
        cust_company.setEditable(true);
        cust_city.setEditable(true);
        cust_state.setEditable(true);
        cust_zip.setEditable(true);
        cust_phone.setEditable(true);
        empl_first.setEditable(true);
        empl_last.setEditable(true);
        empl_dept.select(0);
        empl_dept.enable();

        req_date.setBackground(Color.white);
        cust_first.setBackground(Color.white);
        cust_last.setBackground(Color.white);
        cust_addr.setBackground(Color.white);
        cust_company.setBackground(Color.white);
        cust_city.setBackground(Color.white);
        cust_state.setBackground(Color.white);
        cust_zip.setBackground(Color.white);
        cust_phone.setBackground(Color.white);
        empl_first.setBackground(Color.white);
        empl_last.setBackground(Color.white);
        empl_dept.setBackground(Color.white);

        req_num.setText("");
        req_date.setText("");
        cust_first.setText("");
        cust_last.setText("");
        cust_addr.setText("");
        cust_company.setText("");
        cust_city.setText("");
        cust_state.setText("");
        cust_zip.setText("");
        cust_phone.setText("");
        empl_first.setText("");
        empl_last.setText("");

        add_item_button.hide();
        edit_item_button.hide();
        delete_item_button.hide();
        clear_item_button.hide();
        prod_line_label.hide();
        prod_line.hide();
        prod_id_label.hide();
        prod_id.hide();
        prod_quan_label.hide();
        prod_quan.hide();
        prod_ship_label.hide();
        prod_ship.hide();
        prod_mask.show();

        highlight(req_num);
        message("Ready. [Browse Mode]");
        mode = 0;
    }

    // set_edit()
    final void set_edit(int mode_setting) {
        mode = mode_setting;

        edit_button.disable();
        if (mode_setting != 2) clear_record();
        if (mode_setting == 2) set_modify_mode(false);
        else if ((mode_setting == 3) || (mode_setting == 4)) set_modify_mode(true);

        req_num.setEditable(true);
        req_num.setBackground(Color.black);
        req_num.setForeground(Color.orange);
        req_region.select(0);
        req_region.disable();

        cust_num.setEditable(false);
        cust_num.setBackground(Color.lightGray);
        cust_num.setText("                        ");

        empl_num.setEditable(false);
        empl_num.setBackground(Color.lightGray);
        empl_num.setText("                        ");

        edit_button.setLabel("Add");

        highlight(req_num);
        if (mode == 2) message("Enter record to delete and press return.");
        else if (mode == 4) {
            message("Enter record to edit, or click Add to add a new record.");
            clear_button.disable();
        }
        edit_button.enable();
    }

    // set_edit_item()
    final void set_edit_item(int mode_setting) {
        if (mode_setting == 3) {
            mode = 3;
            edit_item_button.disable();
            confirm_button.setLabel("Confirm Update");
        }
        else if (mode_setting == 2) {
            mode = 2;
            delete_item_button.disable();
            confirm_button.setLabel("Confirm Delete");
        }
        clear_item();

        add_item_button.enable();
        edit_item_button.disable();
        delete_button.enable();
        confirm_button.enable();

        prod_id.setEditable(false);
        prod_quan.setEditable(false);
        prod_ship.setEditable(false);

        prod_id.setBackground(Color.lightGray);
        prod_quan.setBackground(Color.lightGray);
        prod_ship.setBackground(Color.lightGray);

        prod_id.setText("                        ");
        prod_quan.setText("                        ");
        prod_ship.setText("                        ");

        prod_line.setEditable(true);
        highlight(prod_line);
    }

    // set_editing()
    final void set_editing() {
        mode = 4;
        delete_item_button.disable();

        message("Click on Add Item to continue adding items.");

        prod_line.setEditable(false);
        prod_line.setBackground(Color.lightGray);

        prod_id.setEditable(true);
        prod_quan.setEditable(true);
        prod_ship.setEditable(true);

        prod_id.setBackground(Color.white);
        prod_quan.setBackground(Color.white);
        prod_ship.setBackground(Color.white);

        prod_id.requestFocus();
        prod_id.selectAll();
    }

    // set_modify_mode()
    final void set_modify_mode(boolean modify_state) {
        record_saved = false;

        if (modify_state) {
            disable_buttons();
            delete_button.hide();
            save_button.show();
            edit_button.enable();
            edit_button.show();
            edit_button.setLabel("Edit");
            browse_button.enable();
            browse_button.show();
            add_button.hide();
            clear_button.enable();

            add_item_button.show();
            edit_item_button.show();
            delete_item_button.show();
            clear_item_button.show();
            clear_item_button.enable();
            prod_line_label.show();
            prod_line.show();
            prod_line.setEditable(false);
            prod_line.setBackground(Color.lightGray);
            prod_id_label.show();
            prod_id.show();
            prod_quan_label.show();
            prod_quan.show();
            prod_ship_label.show();
            prod_ship.show();
            prod_mask.hide();
        }
        first_button.disable();
        next_button.disable();

        if (mode == 2) clear_button.setLabel("Cancel Delete");
        else clear_button.setLabel("Clear Record");
        req_total.hide();
        record_label.hide();
        req_current.hide();

        req_num.setEditable(false);
        req_date.setEditable(false);
        req_region.select(0);
        req_region.enable();
        cust_num.setEditable(true);
        cust_first.setEditable(false);
        cust_last.setEditable(false);
        cust_addr.setEditable(false);
        cust_company.setEditable(false);
        cust_city.setEditable(false);
        cust_state.setEditable(false);
        cust_zip.setEditable(false);
        cust_phone.setEditable(false);
        empl_num.setEditable(true);
        empl_first.setEditable(false);
        empl_last.setEditable(false);
        empl_dept.select(0);
        empl_dept.disable();

        req_num.setBackground(Color.lightGray);
        req_num.setForeground(Color.red);
        req_date.setBackground(Color.lightGray);
        cust_num.setBackground(Color.white);
        cust_first.setBackground(Color.lightGray);
        cust_last.setBackground(Color.lightGray);
        cust_addr.setBackground(Color.lightGray);
        cust_company.setBackground(Color.lightGray);
        cust_city.setBackground(Color.lightGray);
        cust_state.setBackground(Color.lightGray);
        cust_zip.setBackground(Color.lightGray);
        cust_phone.setBackground(Color.lightGray);
        empl_num.setBackground(Color.white);
        empl_first.setBackground(Color.lightGray);
        empl_last.setBackground(Color.lightGray);
        empl_dept.setBackground(Color.lightGray);

        req_num.setText("                        ");
        req_date.setText("                        ");
        cust_num.setText("                        ");
        cust_first.setText("                        ");
        cust_last.setText("                        ");
        cust_addr.setText("                        ");
        cust_company.setText("                        ");
        cust_city.setText("                        ");
        cust_state.setText("                        ");
        cust_zip.setText("                        ");
        cust_phone.setText("                        ");
        empl_num.setText("                        ");
        empl_first.setText("                        ");
        empl_last.setText("                        ");

        if (mode != 2) { message("Ready. [Add Mode]"); req_region.requestFocus(); }
    }

    // show_information()
    final void show_information() {
        SQL_string_asc = generate_SQL(mode, false);

        counter = 0;
        current_pos = 0;

        try {
            message("Counting records...");
            rs = generate_ResultSet(generate_SQL(mode, true), rs, stmt, true);
        }
        catch (Exception filter_error) {
            message("Exception caught!  Could not create counter!");
            sleep();
        }

        try { counter = rs.getInt(1); rs.close(); }
        catch (SQLException get_error) {
            message("Exception caught!  Could not get grab int for max record!");
            sleep();
        }

        req_total.setText(String.valueOf(counter));

        if (counter != 0) {
            message("Number of records found in result set...  "+String.valueOf(counter)+".");
            rs = generate_ResultSet(SQL_string_asc, rs, stmt, false);
            move(1);
            message("Ready. [Browse mode]");

            if (mode == 2) {
                message("Click on delete if you are sure you want to delete this record.");
                delete_button.enable();
                delete_button.requestFocus();
            }
        }
        else {
            req_current.setText(String.valueOf(current_pos));
            message("No hits found.");
            clear_record();
        }

    }


    // show_order_list()
    final void show_order_list() {
        String padding = new String("                                                   ");
        String temp_string;
        BigDecimal subtotal = new BigDecimal ("0");
        BigDecimal sub_temp = new BigDecimal ("0");

        try {
            String select = "SELECT sales_order_items.id, line_id, prod_id, sales_order_items.quantity, ship_date, name, description, size, color, unit_price";
            String from = " FROM sales_order_items, product";
            String where = " WHERE product.id = prod_id AND sales_order_items.id = '"+Integer.valueOf(req_num.getText().trim()).intValue()+"'";

            message("Grabbing ordered items");
            rs = generate_ResultSet(select + from + where, rs, stmt, false);
            message("Displaying ordered items");

            order_list.setText("");
            while (rs.next()) {
                message("Displaying ordered items");

                temp_string = new String(rs.getString(2).concat(padding));
                temp_string = temp_string.substring(0, 4);

                BigDecimal price_temp = new BigDecimal(rs.getBigDecimal(10, 2).doubleValue());
                BigDecimal quantity_temp = new BigDecimal(rs.getString(4));

                order_list.appendText(temp_string+"   $"+price_temp.toString()+"\t     "
                    +rs.getString(4)+"\t     "+rs.getString(5)+"\t"
                    +rs.getString(6)+" - "+rs.getString(7)+"\n");
                order_list.appendText("\t\t\t\t\tSize: "+rs.getString(8)+" - Color: "
                    +rs.getString(9)+"\n");
                message("Calculating subtotal...");

                sub_temp = price_temp.multiply(quantity_temp);
                subtotal = subtotal.add(sub_temp);
            }

            order_list.appendText("--------------------------------------------------------------------------\n");
            order_list.appendText("\t\t\t\t\tSubtotal: $"+subtotal+"\n");

            message("Ready.");

            rs.close();
        }
        catch (SQLException order_list_error) {
            message("Exception caught!  Could not continue with order listing!");
        }
    }

    // sleep();
    final void sleep() {
        try { server.close(); }
        catch (SQLException close_server_error) { message("Exception caught!  Could not close server!!!"); }
        disable_buttons();
        for (int i = 0; i < 3; i++) i = 1;
    }
/*
    // start()
    public void start() {
	    if (kicker == null) {
            kicker = new Thread(this);
	        kicker.setPriority(kicker.MIN_PRIORITY);
	        kicker.start();
        }
    }

    // stop()
	public void stop() {
	    close_server();
		kicker = null;
	}

    // run()
    public void run() {
        int cntr = 0;

        while (kicker != null) {
            cntr++;
            if ((cntr%1000) == 0) first = true;
            try {kicker.sleep(100);} catch (InterruptedException e) {}
        }
    }
*/
    // update()
    public void update(Graphics g) {
        g.clearRect(0, 0, 540, 540);
        paint(g);
    }

    // update_delete()
    final void update_delete(boolean delete_item, boolean delete_record) {
        boolean is_missing = true;


        if ((req_num.getText().trim().length() != 0) && (prod_line.getText().trim().length() != 0)
            && (prod_id.getText().trim().length() != 0) && (prod_quan.getText().trim().length() != 0)
            && (prod_ship.getText().trim().length() != 0)) { is_missing = false; }
        else { message("Cannot save!  Empty fields were detected in item!"); }

        if (delete_item) {
            String delete_item_SQL = "DELETE FROM sales_order_items WHERE id = " + req_num.getText().trim() + " AND line_id = " + prod_line.getText().trim();

            if (!is_missing) {
                message("Deleting item...");

                try {
                    stmt.executeUpdate(delete_item_SQL);
                    message("Deleting item... done.");
                    update_prod_line(true);
                }
                catch (SQLException delete_error) { message("Caution! Could not delete item!"); }
                show_order_list();
            }
        }
        else if ((req_num.getText().trim().length() != 0) && delete_record) {
            String delete_item_SQL2 = "DELETE FROM sales_order_items WHERE id = " + req_num.getText().trim();
            String delete_record_SQL = "DELETE FROM sales_order WHERE id = " + req_num.getText().trim();

            message("Deleting items in record...");

            try { stmt.executeUpdate(delete_item_SQL2); }
            catch (SQLException delete_item_error) { message("Caution! Could not delete all the items in the record!"); }

            try { stmt.executeUpdate(delete_record_SQL); }
            catch (SQLException delete_record_error) {message("Caution! Could not delete record!"); }

            clear_record();
            set_browse_mode();
        }
        else {
            String update_SQL = "UPDATE sales_order_items SET prod_id = " + prod_id.getText().trim() + ", quantity = " + prod_quan.getText().trim() + ", ship_date = '" + prod_ship.getText().trim() + "' WHERE id = " + req_num.getText().trim() + " AND line_id = " + prod_line.getText().trim();

            if (!is_missing) {
                message("Updating item...");

                try {
                    stmt.executeUpdate(update_SQL);
                    message("Item updated.");
                    update_prod_line(true);
                }
                catch (SQLException save_error) { message("Problem with SQL Statement!  No records were inserted!"); }
                show_order_list();
            }
        }
    }

    // update_prod_line()
    final void update_prod_line(boolean get_last) {
        String count_SQL = "SELECT max(line_id) FROM sales_order_items WHERE id = " + req_num.getText().trim();

        mode = 1;

        if (get_last) {
            rs = generate_ResultSet(count_SQL, rs, stmt, true);
            try { current_item_pos = rs.getInt(1) + 1; rs.close(); }
            catch (SQLException get_error) { message("Exception caught!  Could not grab last item!"); }
            clear_item();
        }

        add_item_button.disable();
        edit_item_button.enable();
        delete_item_button.enable();
        confirm_button.disable();
        prod_line.setEditable(false);
        prod_line.setBackground(Color.lightGray);
        prod_line.setText(String.valueOf(current_item_pos));
        prod_id.setEditable(true);
        prod_id.setBackground(Color.white);
        prod_quan.setEditable(true);
        prod_quan.setBackground(Color.white);
        prod_ship.setEditable(true);
        prod_ship.setBackground(Color.white);
        highlight(prod_id);
    }

    // handleEvent()
    public boolean handleEvent(Event event) {
        Object evt = event.target;
        int evtx = event.x, evty = event.y;

        showStatus("Symantec Corp., 1996");
        if ((mode == 1) || (mode == 3) || (mode == 4)) {
            if ((evtx > 0 && evtx < 125) && (evty > 75 && evty < 95)) status_bar.setText("Switch to Browse Mode to look up record(s).");
            else if ((evtx > 127 && evtx < 182) && (evty > 75 && evty < 95)) status_bar.setText("Delete/Save this record.");
            else if ((evtx > 184 && evtx < 246) && (evty > 75 && evty < 95)) status_bar.setText("Switch between [Add new record]/[Edit existing record].");
            else if ((evtx > 0 && evtx < 80) && (evty > 516 && evty < 536)) status_bar.setText("Add current item to order list.");
            else if ((evtx > 88 && evtx < 186) && (evty > 516 && evty < 536)) status_bar.setText("Choose item in order list to update.");
            else if ((evtx > 196 && evtx < 303) && (evty > 516 && evty < 536)) status_bar.setText("Choose item to delete off of order list.");
            else if ((evtx > 313 && evtx < 433) && (evty > 516 && evty < 536)) status_bar.setText("Concur that you want to save item to order list.");
            else if ((evtx > 443 && evtx < 540) && (evty > 516 && evty < 536)) status_bar.setText("Clear current item entry.");
            else status_bar.setText(current_status_bar);
        }
        else if ((mode == 0) || (mode == 2)) {
            if ((evtx > 0 && evtx < 125) && (evty > 75 && evty <95)) status_bar.setText("Switch to Add/Edit Mode to modify record(s).");
            else if ((evtx > 127 && evtx < 182) && (evty > 75 && evty < 95)) status_bar.setText("Choose record to delete.");
            else if ((evtx > 184 && evtx < 214) && (evty > 75 && evty < 95)) status_bar.setText("Go to the first record in this set.");
            else if ((evtx > 216 && evtx < 246) && (evty > 75 && evty < 95)) status_bar.setText("Go to the next record in this set.");
            else if ((evtx > 248 && evtx < 344) && (evty > 75 && evty < 95)) status_bar.setText("Search the database given the information that you have entered.");
            else status_bar.setText(current_status_bar);
        }
        if ((evtx > 346 && evtx < 442) && (evty > 75 && evty < 95) && (mode != 2)) status_bar.setText("Clear current record entry.");
        else if ((evtx > 346 && evtx < 442) && (evty > 75 && evty < 95) && (mode == 2)) status_bar.setText("Continue with browsing.");
        if ((evtx > 444 && evtx < 540) && (evty > 75 && evty < 95)) status_bar.setText("Quit this program.");


        if ((event.id == Event.ACTION_EVENT) && (evt instanceof Button)) {
            if (evt == add_button) { set_add_mode(); }
            else if (evt == add_item_button) {
                if (mode == 1) {
                    save_item_info();
                    add_item_button.disable();
                    highlight(prod_id);
                }
                else if ((mode >= 2) || (mode <= 4)) { update_prod_line(true); }
            }
            else if (evt == browse_button) { clear_record(); set_browse_mode(); }
            else if (evt == clear_button) {
                clear_record();
                if (mode == 1) req_region.requestFocus();
                else highlight(req_num);
            }
            else if (evt == clear_item_button) { clear_item(); }
            else if (evt == confirm_button) {
                if ((mode == 4) || (mode == 1)) { update_delete(false, false); }
                else if (mode == 2) { update_delete(true, false); }
            }
            else if (evt == delete_button) {
                if (mode == 2) { update_delete(false, true); }
                else { set_edit(2); delete_button.disable(); }
            }
            else if (evt == delete_item_button) {
                message("Click on Add Item to continue adding items.");
                set_edit_item(2);
            }
            else if (evt == edit_button) {
                if ((mode == 4) || (mode == 3)){ set_add_mode(); }
                else if (mode == 1) { set_edit(4); }
            }
            else if (evt == edit_item_button) {
                message("Click on Add Item to continue adding items.");
                set_edit_item(3);
            }
            else if (evt == first_button) {
                current_pos = (counter - counter);
                nav_buttons(false, false);
                rs= generate_ResultSet(SQL_string_asc, rs, stmt, false);
                move(1);
            }
            else if (evt == next_button) {
                current_pos++;
                nav_buttons(false, false);
                move(2);
            }
            else if (evt == save_button) {
                save_info();
                save_button.disable();
                add_item_button.enable();
                add_item_button.requestFocus();
            }
            else if (evt == search_button) { show_information(); }
            else { InterruptedException e; return false; }
            return true;
        }
        else if ((event.id == Event.KEY_PRESS) && ((event.key == 9) || (event.key == 10))) {
            if ((mode == 4) && (event.key == 9)) {
                if (evt == prod_id) { get_prod_info(); }
                else if (evt == prod_quan) { prod_ship.requestFocus(); prod_ship.selectAll(); }
                else if (evt == prod_ship) { prod_id.requestFocus(); prod_id.selectAll(); }
            }
            else if ((mode == 4) && (event.key == 10)) {
                if (evt == req_num) { get_edit_info(); }
                else if (evt == prod_id) { get_prod_info(); }
                else if (evt == prod_quan) { prod_ship.requestFocus(); prod_ship.selectAll(); }
                else if (evt == prod_ship) {
                    message("Please confirm requested action...");
                    confirm_button.enable();
                    confirm_button.requestFocus();
                }
                else if (evt == confirm_button) { update_delete(false, false); }
            }
            else if ((mode == 3) && (event.key == 10)) { get_prod_item(); }
            else if ((mode == 2) && (event.key == 10)) {
                if (evt == prod_line) { get_prod_item(); }
                else if (evt == req_num) { show_information(); }
                else if (evt == confirm_button) {  update_delete(true, false); }
            }
            else if ((mode == 1) && (event.key == 10)) {
                if (evt == req_region) { highlight(cust_num); }
                else if (evt == cust_num) { get_cust_info(); }
                else if (evt == empl_num) { get_empl_info(); }
                else if (evt == prod_line) { get_prod_item(); set_editing(); }
                else if (evt == prod_id) { get_prod_info(); }
                else if (evt == prod_quan) { highlight(prod_ship); }
                else if (evt == prod_ship) {
                    if (record_saved) {
                        message("Press return or click on the Add Item button to save item.");
                        add_item_button.enable();
                        add_item_button.requestFocus();
                    }
                    else {
                        message("Press return or click on the Save button to save.");
                        save_button.enable();
                        save_button.requestFocus();
                    }
                }
                else if (evt == add_item_button) {
                    save_item_info();
                    add_item_button.disable();
                    highlight(prod_id);
                }
                else if (evt == edit_item_button) {
                    message("Click on Add Item to continue adding items.");
                    set_edit_item(3);
                }
                else if (evt == save_button) {
                    save_info();
                    save_button.disable();
                    add_item_button.enable();
                    add_item_button.requestFocus();
                }
            }
            else if ((mode == 1) && (event.key == 9)) {
                if (evt == req_region) { highlight(cust_num); }
                else if (evt == cust_num) { get_cust_info(); }
                else if (evt == empl_num) { get_empl_info(); }
                else if (evt == prod_line) { get_prod_item(); set_editing(); }
                else if (evt == prod_id) { get_prod_info(); }
                else if (evt == prod_quan) { highlight(prod_ship); }
                else if (evt == prod_ship) { req_region.requestFocus(); }
            }
            else if ((mode == 0) && ((event.key == 9) || (event.key == 10))) {
                if (evt == req_num) { req_date.requestFocus(); }
                else if (evt == req_date) { req_region.requestFocus(); }
                else if (evt == req_region) { cust_num.requestFocus(); }
                else if (evt == cust_num) { cust_first.requestFocus(); }
                else if (evt == cust_first) { cust_last.requestFocus(); }
                else if (evt == cust_last) { cust_addr.requestFocus(); }
                else if (evt == cust_addr) { cust_city.requestFocus(); }
                else if (evt == cust_city) { cust_state.requestFocus(); }
                else if (evt == cust_state) { cust_zip.requestFocus(); }
                else if (evt == cust_zip) { cust_company.requestFocus(); }
                else if (evt == cust_company) { cust_phone.requestFocus(); }
                else if (evt == cust_phone) { empl_num.requestFocus(); }
                else if (evt == empl_num) { empl_first.requestFocus(); }
                else if (evt == empl_first) { empl_last.requestFocus(); }
                else if (evt == empl_last) { empl_dept.requestFocus(); }
                else if (evt == empl_dept) { req_num.requestFocus(); }
            }
            return true;
        }
        else if ((event.id == Event.MOUSE_DOWN) && (evt == this)) { update(this.getGraphics()); return true; }
        else if (event.id == Event.WINDOW_DESTROY) { close_server(); System.exit(0); return true; }
        else if (event.id == Event.WINDOW_MOVED) { update(this.getGraphics()); return true; }
        else return super.handleEvent(event);
    }

    // Global Objects
    private boolean first = false, good_connection = false, record_saved = false;
    private int counter = 0, current_pos = 0, current_item_pos = 0, mode = 0;
    // mode: 0 = browse, 1 = add, 2 = delete item, 3 = update item, 4 = editing items
    private Color current_color = Color.white;
    private Font cour_reg_font = new Font("Courier", Font.PLAIN, 12);
    private Font hel_reg_font = new Font("Helvetica", Font.PLAIN, 12);
    private Font hel_reg_11_font = new Font("Helvetica", Font.PLAIN, 11);
    private Font hel_reg_14_font = new Font("Helvetica", Font.PLAIN, 14);
    private Font hel_bold_16_font = new Font("Helvetica", Font.BOLD, 16);
    private Font company_font = new Font("Helvetica", Font.BOLD, 26);
    private String current_status_bar;

    // Instantiate UI and Thread Objects
    Button browse_button, add_button, delete_button, edit_button, save_button, clear_button;
    Button first_button, next_button, search_button, quit_button;
    Button add_item_button, edit_item_button, delete_item_button, clear_item_button, confirm_button;

    Choice empl_dept, req_region;

    Label record_label;
    Label req_date_label, req_fin_label, req_region_label;
    Label cust_first_label, cust_last_label, cust_addr_label, cust_city_label;
    Label cust_state_label, cust_zip_label, cust_phone_label, cust_company_label;
    Label empl_first_label, empl_last_label, empl_dept_label;
    Label prod_line_label, prod_id_label, prod_quan_label, prod_ship_label;

    TextArea order_list;

    TextField prod_header, req_total, req_current;
    TextField req_num, req_date, req_fin, status_bar;
    TextField cust_num, cust_first, cust_last, cust_addr, cust_city;
    TextField cust_state, cust_zip, cust_phone, cust_company;
    TextField empl_num, empl_first, empl_last;
    TextField prod_line, prod_id, prod_quan, prod_ship, prod_mask;

    Thread kicker = null;

    // JDBC Objects
    Connection server;
    DriverManager dmanager;
    Driver dbaw_driver;
    ResultSet rs;
    Statement stmt;
    Properties props = new Properties();
    private String SQL_string_asc;
    private String dbaw_driver_name = "symantec.itools.db.jdbc.Driver";
    private String server_url = "jdbc:dbaw://localhost:8889/Watcom/SQL Anywhere 5.0 Sample/SQL Anywhere";
}